/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.certsecurity.custom.ejb;

import java.security.Principal;
import java.security.acl.Group;
import java.util.Properties;
import javax.annotation.Resource;
import javax.naming.AuthenticationException;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.*;
import javax.naming.ldap.Control;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.LdapName;
import javax.security.auth.x500.X500Principal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * <p>Easy LDAP EJB that let us search a user in the ldap repository, check his
 * password performing a bind and search the membership of this user to 
 * groups (static and dynamic groups). It is implemented as a singleton EJB.</p>
 * 
 * <p>All the parameters are resources that can be annotated or be overwritten
 * via xml file. The parameters are typical ldap parameters (url, bind user and
 * password, default branches,...)</p>
 * 
 * @author ricky
 */
@javax.inject.Named
@javax.ejb.Singleton
public class LdapEjb {

    /**
     * Pattern to use in filters to be substituted by the name
     */
    final static protected String NAME_SUBSTITUTION_PATTERN = "%s";
    /**
     * attributes that controls membership (in both types of group)
     */
    final static protected String[] GROUP_MEMBERSHIP_ATTRIBUTES = new String[] {"uniqueMember", "memberUrl"};
    /**
     * logger for the class
     */
    static private final Logger logger = LoggerFactory.getLogger(LdapEjb.class);
    
    @Resource
    private String dirUrl = "ldap://localhost:1389";
    @Resource
    private String userBaseDn = "ou=people,dc=example,dc=com";
    @Resource
    private String userSearchFilter = "(&(uid=%s)(objectclass=inetorgperson))";
    @Resource
    private String groupBaseDn = "ou=groups,dc=example,dc=com";
    @Resource
    private String groupSearchFilter = "(&(cn=%s)(|(objectclass=groupofuniquenames)(objectclass=groupofurls)))";
    @Resource
    private String objectBaseFilter = "(objectclass=*)";
    @Resource
    private String bindDN = "cn=Directory Manager";
    @Resource
    private String bindPassword = "kiosko_00";
    @Resource
    private String usePool = "true";
    @Resource
    private String protocol = "plain ssl";
    @Resource
    private String userPasswordAttribute = "userPassword";
    Properties ldapProps = null;

    /**
     * Empty Constructor
     */
    public LdapEjb() {
        logger.debug("entring LdapEjb");
        ldapProps = new Properties();
        ldapProps.setProperty(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        ldapProps.setProperty("java.naming.factory.object", "com.sun.jndi.ldap.obj.LdapGroupFactory");
        ldapProps.setProperty("java.naming.factory.state", "com.sun.jndi.ldap.obj.LdapGroupFactory");
        ldapProps.setProperty(Context.PROVIDER_URL, dirUrl);
        ldapProps.put(Context.SECURITY_AUTHENTICATION, "simple");
        ldapProps.put(Context.SECURITY_PRINCIPAL, bindDN);
        ldapProps.put(Context.SECURITY_CREDENTIALS, bindPassword);
        ldapProps.put(Context.SECURITY_PROTOCOL, protocol);
        logger.debug("exiting LdapEjb");
    }

    //
    // GETTERS & SETTERS
    //
    
    /**
     * Getter for the URL of the ldap. Example: "ldap://localhost:389"
     * @return The ldap url 
     */
    public String getDirUrl() {
        return dirUrl;
    }

    /**
     * Setter for the URL of the ldap. Example: "ldap://localhost:389"
     * @param dirUrl  The new ldap url
     */
    public void setDirUrl(String dirUrl) {
        this.dirUrl = dirUrl;
        ldapProps.setProperty(Context.PROVIDER_URL, dirUrl);
        ldapProps.put(Context.SECURITY_AUTHENTICATION, "simple");
    }

    /**
     * Getter for group base DN. Example: "ou=groups,dc=sample,dc=com"
     * @return The base dn to search groups
     */
    public String getGroupBaseDn() {
        return groupBaseDn;
    }

    /**
     * Setter for group base DN. Example: "ou=groups,dc=sample,dc=com"
     * @param groupBaseDn The new base dn for groups
     */
    public void setGroupBaseDn(String groupBaseDn) {
        this.groupBaseDn = groupBaseDn;
    }

    /**
     * Getter for group search filter. Example: "(&(cn=%s)(objectclass=groupofuniquenames))"
     * You can use "%s" as the group name.
     * @return the group search filter
     */
    public String getGroupSearchFilter() {
        return groupSearchFilter;
    }

    /**
     * Setter for group search filter. Example: "(&(cn=%s)(objectclass=groupofuniquenames))"
     * You can use "%s" as the group name.
     * @param groupSearchFilter the new group search filter
     */
    public void setGroupSearchFilter(String groupSearchFilter) {
        this.groupSearchFilter = groupSearchFilter;
    }

    /**
     * Getter for user search filter. Example: "(&(cn=%s)(objectclass=inetorgperson))"
     * You can use "%s" as the user name.
     * @return The user search filter
     */
    public String getUserSearchFilter() {
        return userSearchFilter;
    }

    /**
     * Setter for user search filter. Example: "(&(cn=%s)(objectclass=inetorgperson))"
     * You can use "%s" as the user name.
     * @param userSearchFilter The new user search filter
     */
    public void setUserSearchFilter(String userSearchFilter) {
        this.userSearchFilter = userSearchFilter;
    }

    /**
     * Getter of the filter used when a base query is performed (a query
     * over a single entry that should be always retrieved). EXample: "(objectclass=*)"
     * @return The base scope filter
     */
    public String getObjectBaseFilter() {
        return objectBaseFilter;
    }

    /**
     * Setter of the filter used when a base query is performed (a query
     * over a single entry that should be always retrieved). EXample: "(objectclass=*)"
     * @param objectBaseFilter The new base scope filter
     */
    public void setObjectBaseFilter(String objectBaseFilter) {
        this.objectBaseFilter = objectBaseFilter;
    }

    /**
     * Getter for user base DN. Example: "ou=people,dc=sample,dc=com"
     * @return The base used to user queries
     */
    public String getUserBaseDn() {
        return userBaseDn;
    }

    /**
     * Setter for user base DN. Example: "ou=people,dc=sample,dc=com"
     * @param userBaseDn The new user base
     */
    public void setUserBaseDn(String userBaseDn) {
        this.userBaseDn = userBaseDn;
    }

    /**
     * Getter of the DN to use as connection user.
     * @return The dn used for connections
     */
    public String getBindDN() {
        return bindDN;
    }

    /**
     * Setter for the DN to use as connection user.
     * @param bindDN  The new user DN
     */
    public void setBindDN(String bindDN) {
        this.bindDN = bindDN;
        ldapProps.put(Context.SECURITY_PRINCIPAL, bindDN);
    }

    /**
     * Getter for the connection password
     * @return the password used to connect to the ldap
     */
    public String getBindPassword() {
        return bindPassword;
    }

    /**
     * Setter for the connection password
     * @param bindPassword  The new password
     */
    public void setBindPassword(String bindPassword) {
        this.bindPassword = bindPassword;
        ldapProps.put(Context.SECURITY_CREDENTIALS, bindPassword);
    }

    /**
     * Getter for use pool or not in the connection. Rest of properties must be
     * provided via java properties (see http://download.oracle.com/javase/jndi/tutorial/ldap/connect/config.html)
     * @return Whether pooling is used for connection or not
     */
    public String isUsePool() {
        return usePool;
    }

    /**
     * Setter for use pool or not in the connection. Rest of properties must be
     * provided via java properties (see http://download.oracle.com/javase/jndi/tutorial/ldap/connect/config.html)
     * @param usePool 
     */
    public void setUsePool(String usePool) {
        this.usePool = usePool;
        ldapProps.setProperty("com.sun.jndi.ldap.connect.pool", usePool);
    }

    /**
     * Getter for the protocol used (plain, ssl,...)
     * @return The protocol used
     */
    public String getProtocol() {
        return protocol;
    }

    /**
     * Setter for the protocol used (plain, ssl,...)
     * @param protocol The new protocol
     */
    public void setProtocol(String protocol) {
        this.protocol = protocol;
        ldapProps.put(Context.SECURITY_PROTOCOL, protocol);
    }

    /**
     * Getter for the userPassword attribute.
     * @return The attribute used for passwords
     */
    public String getUserPasswordAttribute() {
        return userPasswordAttribute;
    }

    /**
     * Seeter for userPassword attribute.
     * @param userPasswordAttribute The new attribute for passwords
     */
    public void setUserPasswordAttribute(String userPasswordAttribute) {
        this.userPasswordAttribute = userPasswordAttribute;
    }

    //
    // METHODS
    //
    
    /**
     * Method to substitute the "target" with the "value". Used for the "%s".
     * @param sb The String buffer
     * @param target The target string to substitute
     * @param value The value to put instead of target
     * @return 
     */
    private static StringBuilder substitute(StringBuilder sb, String target, String value) {
        int i = sb.indexOf(target);
        while (i >= 0) {
            sb.replace(i, i + target.length(), value);
            i = sb.indexOf(target);
        }
        return sb;
    }

    /**
     * Return a clone of the ldap properties constructed via resources.
     * @return The ldap properties to connect to the ldap
     */
    private Properties getLdapBindProps() {
        return (Properties) ldapProps.clone();
    }
    
    /**
     * Method that binds a user DN against the ldap. It returns true if ok
     * false otherwise. 
     * @param bindDN The user dn to bond
     * @param password The password of the user
     * @return The status of the user
     */
    public UserBindResult bindAsUser(String bindDN, char[] password) {
        logger.debug("entering bindAsUser {}", bindDN);
        UserBindResult result;
        Properties p = getLdapBindProps();
        p.setProperty("com.sun.jndi.ldap.connect.pool", "false");
        LdapContext ctx = null;
        Control[] ctrls = null;
        try {
            ctx = new InitialLdapContext(p, null);
            // change connection to the user
            ctx.addToEnvironment(Context.SECURITY_PRINCIPAL, bindDN);
            ctx.addToEnvironment(Context.SECURITY_CREDENTIALS, new String(password));
            ctx.reconnect(null);
            ctrls = ctx.getResponseControls();
            result = UserBindStatus.checkOkResponse(ctrls);
        } catch (AuthenticationException e) {
            logger.info("AuthException binding the user", e);
            try {
                result = UserBindStatus.checkErrorResponse(ctrls, e.getMessage());
            } catch(Exception ex) {
                logger.info("Cannot check the status", e);
                result = UserBindStatus.getDefaultError();
            }
        } catch (Exception e) {
            logger.error("Error binding user", e);
            result = UserBindStatus.getDefaultError();
        } finally {
            if (ctx != null) {
                try {
                    ctx.close();
                } catch (NamingException e) {
                }
            }
        }
        logger.debug("exiting: {}", result);
        return result;
    }
    
    /**
     * Change the user password with the self users. Right now the method
     * used a normal replace operation, not the extended operation.
     * @param bindDN The DN of the user
     * @param current The current password to login
     * @param newone The new password to set
     * @throws NamingException Some error performing the operation
     */
    public void changeUserPassword(String bindDN, String current, String newone
            ) throws NamingException {
        logger.debug("entering changeUserPassword {}", bindDN);
        Properties p = getLdapBindProps();
        p.put(Context.SECURITY_PRINCIPAL, bindDN);
        p.put(Context.SECURITY_CREDENTIALS, current);
        p.setProperty("com.sun.jndi.ldap.connect.pool", "false");
        LdapContext ctx = null;
        try {
            ctx = new InitialLdapContext(p, null);
            // construct a replace for the user password attribute
            Attribute a = new BasicAttribute(userPasswordAttribute);
            a.add(newone);
            ModificationItem[] mods = new ModificationItem[1];
            mods[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, a);
            ctx.modifyAttributes(bindDN, mods);
        } finally {
            if (ctx != null) {
                try {
                    ctx.close();
                } catch (NamingException e) {
                }
            }
        }
        logger.debug("exiting.");
    }

    /**
     * Search a unique object in the LDAP. The object is returned via
     * LdapEnntry object. Scope is fixed to SebTree. The method returns the
     * first entry if more results are found.
     * 
     * @param ctx The context to use
     * @param baseDN The baseDN to perform the query
     * @param filter The filter to apply
     * @param attrs The attributes to return
     * @return The ldap entry or null if not found
     */
    public LdapEntry objectSearch(DirContext ctx, String baseDN, String filter, String[] attrs) {
        return objectSearch(ctx, baseDN, filter, attrs, SearchControls.SUBTREE_SCOPE);
    }
    
    /**
     * Search a unique object in the LDAP. The object is returned via
     * LdapEnntry object. The method returns the
     * first entry if more results are found.
     * 
     * @param ctx The context to use
     * @param baseDN The baseDN to perform the query
     * @param filter The filter to apply
     * @param attrs The attributes to return
     * @param scope The scope to use in the search
     * @return The ldap entry or null if not found
     */
    public LdapEntry objectSearch(DirContext ctx, String baseDN, String filter, String[] attrs, int scope) {
        logger.debug("Entering objectSearch {}:{}", baseDN, filter);
        LdapEntry result = null;
        NamingEnumeration namingEnum = null;
        SearchControls ctls = new SearchControls();
        ctls.setReturningAttributes(attrs);
        ctls.setSearchScope(scope);
        ctls.setCountLimit(1);
        try {
            namingEnum = ctx.search(baseDN, filter, ctls);
            if (namingEnum.hasMore()) {
                SearchResult sr = (SearchResult) namingEnum.next();
                result = new LdapEntry(sr);
            }
        } catch (Exception e) {
            logger.error("Error objectSearch", e);
        } finally {
            if (namingEnum != null) {
                try {
                    namingEnum.close();
                } catch (Exception ex) {
                }
            }
        }
        logger.debug("Exiting: " + result);
        return result;
    }
    
    /**
     * Method that searches a group using the group name. Obviously it uses
     * the group filter and group base DN defined. This method used 
     * objectSearch internally.
     * 
     * @param groupname The group name to search
     * @param attrs The attributes to return
     * @return The ldapEntry or null
     */
    public LdapEntry findGroup(String groupname, String[] attrs) {
        logger.debug("Entering findGroup {}", groupname);
        // do search for user, substituting %s for username
        LdapEntry result = null;
        StringBuilder sb = new StringBuilder(this.groupSearchFilter);
        substitute(sb, NAME_SUBSTITUTION_PATTERN, groupname);
        // attempt to bind as the user
        LdapContext ctx = null;
        try {
            ctx = new InitialLdapContext(getLdapBindProps(), null);
            result = objectSearch(ctx, this.groupBaseDn, sb.toString(), attrs);
        } catch (Exception e) {
            logger.error("Error in findGroup", e);
        } finally {
            if (ctx != null) {
                try {
                    ctx.close();
                } catch (NamingException e) {
                }
            }
        }
        logger.debug("Exiting: " + result);
        return result;
    }

    /**
     * Method that searches a user using the user name. Obviously it uses
     * the user filter and user base DN defined. This method used 
     * objectSearch internally.
     * 
     * @param username The user name to search
     * @return The ldapEntry or null (not found or wrong password)
     */
    public LdapEntry findUser(String username) {
        logger.debug("Entering findUser {}", username);
        // do search for user, substituting %s for username
        LdapEntry result = null;
        StringBuilder sb = new StringBuilder(this.userSearchFilter);
        substitute(sb, NAME_SUBSTITUTION_PATTERN, username);
        // attempt to bind as the user
        LdapContext ctx = null;
        try {
            ctx = new InitialLdapContext(getLdapBindProps(), null);
            result = objectSearch(ctx, this.userBaseDn, sb.toString(), new String[]{});
            if (result == null) {
                return null;
            }
        } catch (Exception e) {
            logger.error("Error in findUser", e);
        } finally {
            if (ctx != null) {
                try {
                    ctx.close();
                } catch (NamingException e) {
                }
            }
        }
        logger.debug("Exiting: " + result);
        return result;
    }
    
    /**
     * Method that checks if a specified user belongs to a group. For this
     * method is used Sun ldapbp 1.1 library (using GroupOfURLs and GroupOfUniqueNames
     * implementation classes). This method is overridden to check via names or
     * complete DN.
     * 
     * @param username The user  name
     * @param groupname The group name
     * @return true if the user belongs to the group false if not or error
     */
    public boolean isUserInGroup(String username, String groupname) {
        logger.debug("Entering isUserInGroup: {} in {}", username, groupname);
        LdapEntry userEntry = findUser(username);
        if (userEntry == null) {
            return false;
        }
        boolean result = isUserInGroup(userEntry.getDn(), groupname);
        logger.debug("Exiting: " + result);
        return result;
    }
    
    /**
     * Method that checks if a specified user belongs to a group. For this
     * method is used Sun ldapbp 1.1 library (using GroupOfURLs and GroupOfUniqueNames
     * implementation classes). This method is overridden to check via names or
     * complete DN.
     * 
     * @param username The user  name
     * @param groupDn The group DN
     * @return true if the user belongs to the group false if not or error
     */
    public boolean isUserInGroup(String username, LdapName groupDn) {
        logger.debug("Entering isUserInGroup: {} in {}", username, groupDn);
        LdapEntry userEntry = findUser(username);
        if (userEntry == null) {
            return false;
        }
        boolean result = isUserInGroup(userEntry.getDn(), groupDn);
        logger.debug("Exiting: " + result);
        return result;
    }
    
    /**
     * Method that checks if a specified user belongs to a group. For this
     * method is used Sun ldapbp 1.1 library (using GroupOfURLs and GroupOfUniqueNames
     * implementation classes). This method is overridden to check via names or
     * complete DN.
     * 
     * @param userDn The user DN
     * @param groupname The group name
     * @return true if the user belongs to the group false if not or error
     */
    public boolean isUserInGroup(LdapName userDn, String groupname) {
        logger.debug("Entering isUserInGroup: {} in {}", userDn, groupname);
        boolean result = false;
        LdapContext ctx = null;
        StringBuilder sb = new StringBuilder(this.groupSearchFilter);
        substitute(sb, NAME_SUBSTITUTION_PATTERN, groupname);
        try {
            ctx = new InitialLdapContext(getLdapBindProps(), null);
            SearchControls ctls = new SearchControls();
            ctls.setReturningAttributes(GROUP_MEMBERSHIP_ATTRIBUTES);
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            ctls.setReturningObjFlag(true);
            NamingEnumeration e = ctx.search(this.groupBaseDn, sb.toString(), ctls);
            if (e.hasMore()) {
                SearchResult res = (SearchResult) e.next();
                Object searchedObject = res.getObject();
                if (searchedObject instanceof Group) {
                    Group gurls = (Group) searchedObject;
                    Principal x500principal = new X500Principal(userDn.toString());
                    result = gurls.isMember(x500principal);
                }
                // close the object
                if (searchedObject instanceof Context) {
                    ((Context) searchedObject).close();
                }
            }
        } catch (Exception e) {
            logger.error("Error in isUserInGroup", e);
            return false;
        } finally {
            if (ctx != null) {
                try {
                    ctx.close();
                } catch (NamingException e) {
                }
            }
        }
        logger.debug("Exiting: " + result);
        return result;
    }
    
    /**
     * Method that checks if a specified user belongs to a group. For this
     * method is used Sun ldapbp 1.1 library (using GroupOfURLs and GroupOfUniqueNames
     * implementation classes). This method is overridden to check via names or
     * complete DN.
     * 
     * @param userDn The user DN
     * @param groupDn The group DN
     * @return true if the user belongs to the group false if not or error
     */
    public boolean isUserInGroup(LdapName userDn, LdapName groupDn) {
        logger.debug("Entering isUserInGroup: {} in {}", userDn, groupDn);
        boolean result = false;
        LdapContext ctx = null;
        try {
            ctx = new InitialLdapContext(getLdapBindProps(), null);
            SearchControls ctls = new SearchControls();
            ctls.setReturningAttributes(GROUP_MEMBERSHIP_ATTRIBUTES);
            ctls.setSearchScope(SearchControls.OBJECT_SCOPE);
            ctls.setReturningObjFlag(true);
            NamingEnumeration e = ctx.search(groupDn, this.objectBaseFilter, ctls);
            if (e.hasMore()) {
                SearchResult res = (SearchResult) e.next();
                Object searchedObject = res.getObject();
                if (searchedObject instanceof Group) {
                    Group gurls = (Group) searchedObject;
                    Principal x500principal = new X500Principal(userDn.toString());
                    result = gurls.isMember(x500principal);
                }
                // close the object
                if (searchedObject instanceof Context) {
                    ((Context) searchedObject).close();
                }
            }
        } catch (Exception e) {
            logger.error("Error in isUserInGroup", e);
            return false;
        } finally {
            if (ctx != null) {
                try {
                    ctx.close();
                } catch (NamingException e) {
                }
            }
        }
        logger.debug("Exiting: " + result);
        return result;
    }
    
    /**
     * Test method
     * @param args
     * @throws Exception 
     */
    public static void main(String[] args) throws Exception {
        LdapEjb ldap = new LdapEjb();
        LdapEntry sample1 = ldap.findUser("SampleClient1");
        System.err.println(sample1);
        LdapEntry sample2 = ldap.findUser("SampleClient2");
        System.err.println(sample2);
        LdapEntry static1 = ldap.findGroup("StaticGroup1", new String[]{"cn", "objectclass", "uniquemember"});
        System.err.println(static1);
        LdapEntry dyn1 = ldap.findGroup("DynGroupPreSales", new String[]{});
        System.err.println(dyn1);
        
        System.err.println(ldap.isUserInGroup(sample1.getDn(), static1.getDn()));
        System.err.println(ldap.isUserInGroup(sample1.getDn(), static1.getName()));
        System.err.println(ldap.isUserInGroup(sample1.getName(), static1.getDn()));
        System.err.println(ldap.isUserInGroup(sample1.getName(), static1.getName()));
        
        System.err.println(ldap.isUserInGroup(sample2.getDn(), static1.getDn()));
        System.err.println(ldap.isUserInGroup(sample2.getDn(), static1.getName()));
        System.err.println(ldap.isUserInGroup(sample2.getName(), static1.getDn()));
        System.err.println(ldap.isUserInGroup(sample2.getName(), static1.getName()));
        
        System.err.println(ldap.isUserInGroup(sample1.getDn(), dyn1.getDn()));
        System.err.println(ldap.isUserInGroup(sample1.getDn(), dyn1.getName()));
        System.err.println(ldap.isUserInGroup(sample1.getName(), dyn1.getDn()));
        System.err.println(ldap.isUserInGroup(sample1.getName(), dyn1.getName()));
        
        System.err.println(ldap.isUserInGroup(sample2.getDn(), dyn1.getDn()));
        System.err.println(ldap.isUserInGroup(sample2.getDn(), dyn1.getName()));
        System.err.println(ldap.isUserInGroup(sample2.getName(), dyn1.getDn()));
        System.err.println(ldap.isUserInGroup(sample2.getName(), dyn1.getName()));
    }
        
    
}