/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.certsecurity.custom.ejb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.util.Map.Entry;
import javax.naming.InvalidNameException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.LdapName;

/**
 *
 * Simple class that represents any entry in an LDAP repository. It consists
 * in the DN of the entry, the name (the name is the value of the RDN) and the
 * list of attributes read from the entry. The attributes are set into a HashMap
 * of values.
 * 
 * @author ricky
 */
public class LdapEntry {

    /**
     * DN of the entry
     */
    LdapName dn = null;
    /**
     * The name of the entry (value of the RDN)
     * Example: DN: uid=ricky,ou=People,dc=sample,dc=com => name: ricky
     */
    String name = null;
    /**
     * Map for attributes
     */
    Map<String, List<String>> attributes = null;

    /**
     * Empty constructor
     */
    public LdapEntry() {
        dn = null;
        name = null;
        attributes = new HashMap<String, List<String>>();
    }

    /**
     * Constructor via the DN
     * @param dn The DN of the entry
     * @throws InvalidNameException Some error creating the name
     */
    public LdapEntry(String dn) throws InvalidNameException {
        this();
        this.dn = new LdapName(dn);
        this.name = this.dn.getRdn(this.dn.size() - 1).getValue().toString();
    }

    /**
     * Constructor via a SearchResult of a ldap query. The LdapEntry is
     * constructed using the DN and the attributes returned.
     * @param sr The SearchResult obtained by the query
     * @throws InvalidNameException Some error in the name
     * @throws NamingException Some error iterating the result
     */
    public LdapEntry(SearchResult sr) throws InvalidNameException, NamingException {
        this(sr.getNameInNamespace());
        NamingEnumeration<Attribute> neAttrs = (NamingEnumeration<Attribute>) sr.getAttributes().getAll();
        while (neAttrs.hasMore()) {
            Attribute attr = neAttrs.nextElement();
            List<String> values = new ArrayList<String>();
            NamingEnumeration<Object> neValues = (NamingEnumeration<Object>) attr.getAll();
            while (neValues.hasMore()) {
                values.add(neValues.next().toString());
            }
            attributes.put(attr.getID(), values);
        }
    }
    
    //
    // GETTERS & SETTERS
    //

    /**
     * Getter for attributes
     * @return the attributes
     */
    public Map<String, List<String>> getAttributes() {
        return attributes;
    }

    /**
     * Getter for the DN
     * @return the DN
     */
    public LdapName getDn() {
        return dn;
    }

    /**
     * Setter of the DN
     * @param dn the new DN
     */
    public void setDn(LdapName dn) {
        this.dn = dn;
    }

    /**
     * Getter for the name
     * @return The name
     */
    public String getName() {
        return name;
    }

    /**
     * Setter for the name
     * @param name The new name
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     Method that checks if the entry has a specified value of an attribute.
     * The attribute names and values are compared case insensitive.
     * @param attrName The name of the attribute to check
     * @param attrValue The value of the attribute to check
     * @return true if the attribute attrName has a value attrValue
     */
    public boolean containsValue(String attrName, String attrValue) {
        for (String realName: this.attributes.keySet()) {
            if (realName.equalsIgnoreCase(attrName)) {
                for (String value: this.attributes.get(realName)) {
                    if (value.equalsIgnoreCase(attrValue)) {
                        return true;
                    }
                }
                return false;
            }
        }
        return false;
    }
    
    /**
     * String representation. Debug purposes.
     * @return The string representation of an entry
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(this.getClass());
        sb.append(System.getProperty("line.separator"));
        sb.append("dn: ");
        sb.append(dn);
        sb.append(System.getProperty("line.separator"));
        sb.append("name: ");
        sb.append(name);
        sb.append(System.getProperty("line.separator"));
        for (Entry<String,List<String>> entry: attributes.entrySet()) {
            sb.append(entry.getKey());
            sb.append(": ");
            sb.append(entry.getValue());
            sb.append(System.getProperty("line.separator"));
        }
        return sb.toString();
    }

}
