/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.certsecurity.custom.ejb;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import javax.naming.ldap.Control;

/**
 *
 * <p>Now the result of a bind is going to be separated in several status.
 * The status are very easy to understand but not the idea behind this
 * enumerated is the following:</p>
 * 
 * <ul>
 * <li>Any LDAP server sets the special password and blocking status using
 *     controls or error messages in bind responses.</li> 
 * <li>So the status class can have a list of controls and/or a list
 *     of regular expressions. If the bind response has all the controls
 *     defined for the status on success, or all controls or message in the
 *     error response this status is assumed.</li>
 * <li>Statuses are checked in order, if more than one status fulfill the
 *     criteria the first one is returned. There is a list for ok statuses and 
 *     another for error.</li>
 * <li>First controls are checked, then messages.</li>
 * <li>The error messages are checked in substring (the string defined
 *     in the status has to be a substring of the error message returned).</li>
 * <li>No control and no message means the condition is always true. That
 *     means that a status with no control and no message always returns
 *     true no matter the message or the controls returned by the server.</li>
 * <li>There must be a no error, no control for both cases (error and ok list)
 *     and it must be placed at the end of the list. If no match is
 * found the UNKNOWN_ERROR is returned.</li>
 * </ul>
 * 
 * @author ricky
 */
public class UserBindStatus {
    
    //
    // ERROR STATUS
    //
    
    /**
     * User is administratively disabled.
     */
    public static final UserBindStatus DISABLED;
    
    /**
     * User is locked (failed attempts or what ever.
     */
    public static final UserBindStatus LOCKED;
    
    /**
     * User password or account is expired.
     */
    public static final UserBindStatus EXPIRED;
    
    /**
     * Failed bind and no other information given.
     */
    public static final UserBindStatus NOK;
    
    /**
     * Default error used when exceptions thrown. Not in checking lists.
     */
    public static final UserBindStatus UNKNOWN_ERROR;
    
    //
    // OK STATUS
    //
    
    /**
     * The user password must be changed. Because expiration or first time.
     */
    public static final UserBindStatus  PASSWORD_MUST_CHANGE;
    
    /**
     * The user is going to expire soon.
     */
    public static final UserBindStatus WARNING_EXPIRED;
    
    /**
     * User is ok and bind was performed successfully.
     */
    public static final UserBindStatus OK;
    
    /**
     * Ordered list of ok status to check for.
     */
    public static final UserBindStatus[] STATUS_OK_EXECUTION_LIST;
    
    /**
     * Ordered list of ok status to check for.
     */
    public static final UserBindStatus[] STATUS_ERROR_EXECUTION_LIST;
    
    /**
     * Name of the status
     */
    private String name;
    
    /**
     * property that indicates the status is an of error type (only when bind error)
     */
    protected boolean errorType;
    
    /**
     * Controls that marks this state
     */
    protected Set<String> controls;
    
    /**
     * List of strings that marks this status in errorStatus (only for error status)
     */
    protected Set<String> msgs;
    
    /**
     * Empty constructor to create the status. The status is the type (error
     * or ok) and the controls or messages that define it.
     * @param errorType The status is of error or ok type (error means auth exception, ok bind success)
     * @param controls The controls that defines the status
     * @param msgs The messages that defines the status
     */
    protected UserBindStatus(String name, boolean errorType, String[] controls, String[] msgs) {
        this.name = name;
        this.errorType = errorType;
        this.controls = new HashSet(Arrays.asList(controls));
        this.msgs = new HashSet(Arrays.asList(msgs));
    }
    
    /**
     * Copy constructor. No clone done.
     * @param other Other UserBindStatus
     */
    protected UserBindStatus(UserBindStatus other) {
        this.name = other.name;
        this.errorType = other.errorType;
        this.controls = other.controls;
        this.msgs = other.msgs;
    }
    
    //
    // GETTERS
    //

    /**
     * Get the name.
     * @return The name
     */
    public String getName() {
        return name;
    }

    /**
     * Get the controls that defines the status. Any control means this status.
     * @return The set of status
     */
    public Set<String> getControls() {
        return controls;
    }

    /**
     * Is the status a error type. Error means that this status appears only
     * when an AuthenticationException is throws, no error or ok means the
     * status only appears when bind was successfully performed.
     * @return true or false
     */
    public boolean isErrorType() {
        return errorType;
    }

    /**
     * The msgs that defines this status. This messages are only meaningful in
     * error type states (error messages are only returned on error).
     * @return The list of strings that defines this status
     */
    public Set<String> getMsgs() {
        return msgs;
    }
    
    /**
     * Check if any of the controls returned by the LDAP server is in this
     * status object. If any true is returned false otherwise.
     * @param ctrls The controls returned by the server (null is checked)
     * @return true if any matches, false otherwise
     */
    protected boolean checkControls(Control[] ctrls) {
        if (ctrls != null) {
            for (Control c: ctrls) {
                if (this.controls.contains(c.getID())) {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * Check if the exception message returned by the LDAP server matches with
     * any of the messages defined in the status. If any matches true is
     * returned, false otherwise.
     * @param msg The exception message returned by the server
     * @return true is one matches, false if none
     */
    protected boolean checkMessages(String msg) {
        if (msg != null) {
            for (String m: this.msgs) {
                if (msg.indexOf(m) != -1) {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * Perform a check as explained in the class description. If the status
     * has any control or any message matches the error message. A status
     * with no controls and no messages is always true.
     * @param ctrls The controls returned by the server
     * @param msg The error message returned by the server
     * @return true if any matches, false if not
     */
    public boolean check(Control[] ctrls, String msg) {
        // check special case nothing is defined
        if (this.controls.isEmpty() && this.msgs.isEmpty()) {
            return true;
        }
        // now normal test, or consition, first controls, then messages
        return checkControls(ctrls) || checkMessages(msg);
    }
    
    /**
     * Check the controls returned by in a OK response LDAP server against the 
     * list of defined OK status. The status are checked ordered and one of them
     * has to be true (no controls, no messages are always true).
     * @param ctrls
     * @return 
     */
    static public UserBindResult checkOkResponse(Control[] ctrls) {
        for (UserBindStatus s: STATUS_OK_EXECUTION_LIST) {
            if (s.check(ctrls, null)) {
                return new UserBindResult(s, ctrls);
            }
        }
        return getDefaultError();
    }
    
    /**
     * Check the controls and the message returned by the LDAP server in an 
     * error response against the list of defined OK status. The status are 
     * checked ordered and one of them has to be true (no controls, no messages
     * are always true).
     * @param ctrls The controls returned by the LDAP server
     * @param msg The error message returned by the LDAP server
     * @return The user status
     */
    static public UserBindResult checkErrorResponse(Control[] ctrls, String msg) {
        for (UserBindStatus s: STATUS_ERROR_EXECUTION_LIST) {
            if (s.check(ctrls, msg)) {
                return new UserBindResult(s, ctrls);
            }
        }
        return getDefaultError();
    }
    
    /**
     * Return the default error when something strange happens. The
     * UNKNOWN_ERROR result is returned.
     * @return the default status.
     */
    static public UserBindResult getDefaultError() {
        return  new UserBindResult(UNKNOWN_ERROR, null);
    }
    
    static {
        // initialization for NETSCAPE style of locking and expiration
        // Fedora LDAP server, DSEE, OpenDS, OUD, OpenDJ,...
        // TODO: maybe a properties file inicialization????
        // errors
        DISABLED = new UserBindStatus("DISABLED", true, 
                new String[0], 
                new String[] {"because the account has been administrative disabled"});
        LOCKED = new UserBindStatus("LOCKED", true, 
                new String[0], 
                new String[] {"because the account has been locked due to too many failed authentication attempts"});
        EXPIRED = new UserBindStatus("EXPIRED", true, 
                new String[] {"2.16.840.1.113730.3.4.4"}, 
                new String[] {"because that user's password is expired"});
        NOK = new UserBindStatus("NOK", true, 
                new String[0], 
                new String[0]);
        UNKNOWN_ERROR = new UserBindStatus("UNKNOWN_ERROR", true, 
                new String[0], 
                new String[0]);
        // ok
        PASSWORD_MUST_CHANGE = new UserBindStatus("PASSWORD_MUST_CHANGE", false, 
                new String[] {"2.16.840.1.113730.3.4.4"},
                new String[0]);
        WARNING_EXPIRED = new UserBindStatus("WARNING_EXPIRED", false, 
                new String[] {"2.16.840.1.113730.3.4.5"},
                new String[0]);
        OK = new UserBindStatus("OK", false, 
                new String[0], 
                new String[0]);
        // set both list in order of precedence
        STATUS_OK_EXECUTION_LIST = new UserBindStatus[] {
            PASSWORD_MUST_CHANGE, WARNING_EXPIRED, OK
        };
        STATUS_ERROR_EXECUTION_LIST = new UserBindStatus[] {
            DISABLED, LOCKED, EXPIRED, NOK
        };
    }
    
    /**
     * Return string representation.
     * @return 
     */
    @Override
    public String toString() {
        return new StringBuffer()
                .append(this.getClass())
                .append(": ")
                .append(name).toString();
    }
    
    /**
     * Equals by name.
     * @param o The other object
     * @return true if same name
     */
    @Override
    public boolean equals(Object o) {
        if (o instanceof UserBindStatus) {
            return this.name.equals(((UserBindStatus)o).name);
        } else {
            return false;
        }
    }

    /**
     * Hash code over name.
     * @return The hash code
     */
    @Override
    public int hashCode() {
        int hash = 3;
        hash = 47 * hash + (this.name != null ? this.name.hashCode() : 0);
        return hash;
    }
}
